@@ -109,8 +109,11 @@ static int check_parent_dir_perms(pool *p, const char *path) {
   if (res < 0) {
     int xerrno = errno;
 
+    (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+      "unable to check ProxyReverseServers %s directory '%s': %s",
+      path, dir_path, strerror(xerrno));
     pr_log_debug(DEBUG0, MOD_PROXY_VERSION
-      ": unable to stat ProxyReverseServers %s directory '%s': %s",
+      ": unable to check ProxyReverseServers %s directory '%s': %s",
       path, dir_path, strerror(xerrno));
 
     errno = xerrno;
@@ -121,8 +124,12 @@ static int check_parent_dir_perms(pool *p, const char *path) {
       (st.st_mode & S_IWOTH)) {
     int xerrno = EPERM;
 
+    (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+      "unable to use ProxyReverseServers '%s' from world-writable "
+      "directory '%s' (perms %04o): %s", path, dir_path,
+      st.st_mode & ~S_IFMT, strerror(xerrno));
     pr_log_debug(DEBUG0, MOD_PROXY_VERSION
-      ": unable to use ProxyReverseServers %s from world-writable "
+      ": unable to use ProxyReverseServers '%s' from world-writable "
       "directory '%s' (perms %04o): %s", path, dir_path,
       st.st_mode & ~S_IFMT, strerror(xerrno));
 
@@ -144,8 +151,10 @@ static int check_file_perms(pool *p, const char *path) {
   if (res < 0) {
     int xerrno = errno;
 
+    (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+      "unable to check ProxyReverseServers '%s': %s", path, strerror(xerrno));
     pr_log_debug(DEBUG0, MOD_PROXY_VERSION
-      ": unable to lstat ProxyReverseServers '%s': %s", path, strerror(xerrno));
+      ": unable to check ProxyReverseServers '%s': %s", path, strerror(xerrno));
 
     errno = xerrno;
     return -1;
@@ -175,8 +184,11 @@ static int check_file_perms(pool *p, const char *path) {
     if (res < 0) {
       int xerrno = errno;
 
+      (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+        "unable to check ProxyReverseServers '%s': %s", orig_path,
+        strerror(xerrno));
       pr_log_debug(DEBUG0, MOD_PROXY_VERSION
-        ": unable to stat ProxyReverseServers '%s': %s", orig_path,
+        ": unable to check ProxyReverseServers '%s': %s", orig_path,
         strerror(xerrno));
 
       errno = xerrno;
@@ -187,6 +199,9 @@ static int check_file_perms(pool *p, const char *path) {
   if (S_ISDIR(st.st_mode)) {
     int xerrno = EISDIR;
 
+    (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+      "unable to use ProxyReverseServers '%s': %s", orig_path,
+      strerror(xerrno));
     pr_log_debug(DEBUG0, MOD_PROXY_VERSION
       ": unable to use ProxyReverseServers '%s': %s", orig_path,
       strerror(xerrno));
@@ -199,6 +214,9 @@ static int check_file_perms(pool *p, const char *path) {
   if (st.st_mode & S_IWOTH) {
     int xerrno = EPERM;
 
+    (void) pr_log_writefile(proxy_logfd, MOD_PROXY_VERSION,
+      "unable to use world-writable ProxyReverseServers '%s' "
+      "(perms %04o): %s", orig_path, st.st_mode & ~S_IFMT, strerror(xerrno));
     pr_log_debug(DEBUG0, MOD_PROXY_VERSION
       ": unable to use world-writable ProxyReverseServers '%s' "
       "(perms %04o): %s", orig_path, st.st_mode & ~S_IFMT, strerror(xerrno));
@@ -246,7 +264,7 @@ static array_header *reverse_db_parse_uris(pool *p, array_header *uris) {
       continue;
     }
 
-    pconn = proxy_conn_create(p, uri);
+    pconn = proxy_conn_create(p, uri, 0);
     if (pconn == NULL) {
       pr_trace_msg(trace_channel, 9, "skipping malformed URL '%s'", uri);
       continue;
@@ -500,7 +518,7 @@ static array_header *reverse_db_pername_backends_by_json(pool *p,
       per_user ? "user" : "group", path);
 
     PRIVS_ROOT
-    backends = proxy_reverse_json_parse_uris(p, path);
+    backends = proxy_reverse_json_parse_uris(p, path, 0);
     xerrno = errno;
     PRIVS_RELINQUISH
 
@@ -1373,7 +1391,8 @@ static pr_json_array_t *read_json_array(pool *p, pr_fh_t *fh, off_t filesz) {
   return json;
 }
 
-array_header *proxy_reverse_json_parse_uris(pool *p, const char *path) {
+array_header *proxy_reverse_json_parse_uris(pool *p, const char *path,
+    unsigned int flags) {
   register unsigned int i, nelts;
   int count = 0, reached_eol = TRUE, res, xerrno = 0;
   pr_fh_t *fh;
@@ -1487,7 +1506,7 @@ array_header *proxy_reverse_json_parse_uris(pool *p, const char *path) {
     pr_signals_handle();
 
     if (pr_json_array_get_string(p, json, i, &uri) == 0) {
-      pconn = proxy_conn_create(p, uri);
+      pconn = proxy_conn_create(p, uri, flags);
       if (pconn == NULL) {
         pr_trace_msg(trace_channel, 9,
           "skipping malformed URL '%s' found in file '%s'", uri, path);
