@@ -118,16 +118,10 @@ sub config_hash2array {
 sub get_redis_config {
   my $log_file = shift;
 
-  my $redis_server = '127.0.0.1';
-  if (defined($ENV{REDIS_HOST})) {
-    $redis_server = $ENV{REDIS_HOST};
-  }
-
   my $config = {
     RedisEngine => 'on',
     RedisLog => $log_file,
-    RedisServer => "$redis_server:6379",
-    RedisTimeouts => '2000 500',
+    RedisServer => '127.0.0.1:6379',
   };
 
   return $config;
@@ -961,13 +955,7 @@ sub proxy_sql_reverse_config_redis_connect_policy_per_user_by_sql {
   push(@$proxy_config, "ProxyReverseServers sql:/get-user-servers");
   my $nbackends = 1;
 
-  my $redis_server = '127.0.0.1';
-  if (defined($ENV{REDIS_HOST})) {
-    $redis_server = $ENV{REDIS_HOST};
-  }
-
-  # No, the trailing '.' is NOT a typo; it is part of the prefix.
-  push(@$proxy_config, "ProxyDatastore Redis $redis_server.");
+  push(@$proxy_config, "ProxyDatastore Redis 127.0.0.1.");
   my $db_file = File::Spec->rel2abs("$tmpdir/proftpd.db");
 
   # Build up the sqlite3 command to create tables and populate them
@@ -1105,14 +1093,14 @@ EOC
   if ($pid) {
     eval {
       for (my $i = 0; $i < $nbackends+1; $i++) {
-        # Allow for server startup
-        sleep(3);
-        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 0, 2);
+        sleep(2);
+        my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 0, 0);
         $client->login($setup->{user}, $setup->{passwd});
         ftp_list($self, $client, 0);
         $client->quit();
       }
     };
+
     if ($@) {
       $ex = $@;
     }
@@ -1282,14 +1270,11 @@ EOC
   defined(my $pid = fork()) or die("Can't fork: $!");
   if ($pid) {
     eval {
-      # Allow for server startup
-      sleep(2);
-
       for (my $i = 0; $i < $nbackends+1; $i++) {
+        sleep(2);
         my $client = ProFTPD::TestSuite::FTP->new('127.0.0.1', $port, 0, 0);
         $client->login($setup->{user}, $setup->{passwd});
         ftp_list($self, $client);
-        sleep(2);
       }
     };
 
@@ -1432,12 +1417,7 @@ EOS
       'mod_sql.c' => {
         SQLEngine => 'log',
         SQLBackend => 'sqlite3',
-
-        # We provide dummy username/password values, in order to specify
-        # the policy.  In case of chroots, we need to explicitly use
-        # the PERCONNECTION policy.
-        SQLConnectInfo => "$db_file foo bar PERCONNECTION",
-
+        SQLConnectInfo => $db_file,
         SQLLogFile => $setup->{log_file},
         SQLNamedQuery => 'session_start FREEFORM "INSERT INTO proxy_sessions (user, protocol, frontend_ipaddr, local_ipaddr, backend_ipaddr, backend_port, timestamp) VALUES (\'%u\', \'%{protocol}\', \'%a\', \'%L\', \'%{note:mod_proxy.backend-ip}\', %{note:mod_proxy.backend-port}, \'%{iso8601}\')"',
         SQLLog => 'PASS session_start',
@@ -1524,7 +1504,7 @@ EOC
     my $expected = $setup->{user};
     $self->assert($expected eq $login, "Expected '$expected', got '$login'");
 
-    $expected = 'ftp';
+    my $expected = 'ftp';
     $self->assert($expected eq $protocol, "Expected '$expected', got '$protocol'");
 
     $expected = '127.0.0.1';
@@ -1631,12 +1611,7 @@ EOS
       'mod_sql.c' => {
         SQLEngine => 'log',
         SQLBackend => 'sqlite3',
-
-        # We provide dummy username/password values, in order to specify
-        # the policy.  In case of chroots, we need to explicitly use
-        # the PERCONNECTION policy.
-        SQLConnectInfo => "$db_file foo bar PERCONNECTION",
-
+        SQLConnectInfo => $db_file,
         SQLLogFile => $setup->{log_file},
         SQLNamedQuery => 'file_xfer FREEFORM "INSERT INTO proxy_transfers (user, protocol, frontend_ipaddr, local_ipaddr, backend_ipaddr, backend_port, file, timestamp) VALUES (\'%u\', \'%{protocol}\', \'%a\', \'%L\', \'%{note:mod_proxy.backend-ip}\', %{note:mod_proxy.backend-port}, \'%F\', \'%{iso8601}\')"',
         SQLLog => 'RETR,STOR file_xfer',
@@ -1738,7 +1713,7 @@ EOC
     my $expected = $setup->{user};
     $self->assert($expected eq $login, "Expected '$expected', got '$login'");
 
-    $expected = 'ftp';
+    my $expected = 'ftp';
     $self->assert($expected eq $protocol, "Expected '$expected', got '$protocol'");
 
     $expected = '127.0.0.1';
@@ -1838,12 +1813,7 @@ EOS
       'mod_sql.c' => {
         SQLEngine => 'log',
         SQLBackend => 'sqlite3',
-
-        # We provide dummy username/password values, in order to specify
-        # the policy.  In case of chroots, we need to explicitly use
-        # the PERCONNECTION policy.
-        SQLConnectInfo => "$db_file foo bar PERCONNECTION",
-
+        SQLConnectInfo => $db_file,
         SQLLogFile => $setup->{log_file},
         SQLNamedQuery => 'session_start FREEFORM "INSERT INTO proxy_sessions (user, protocol, frontend_ipaddr, local_ipaddr, backend_ipaddr, backend_port, timestamp) VALUES (\'%u\', \'%{protocol}\', \'%a\', \'%L\', \'%{note:mod_proxy.backend-ip}\', %{note:mod_proxy.backend-port}, \'%{iso8601}\')"',
         SQLLog => 'PASS session_start',
@@ -1935,7 +1905,7 @@ EOC
     my $expected = $setup->{user};
     $self->assert($expected eq $login, "Expected '$expected', got '$login'");
 
-    $expected = 'ftp';
+    my $expected = 'ftp';
     $self->assert($expected eq $protocol, "Expected '$expected', got '$protocol'");
 
     $expected = '127.0.0.1';
@@ -2042,12 +2012,7 @@ EOS
       'mod_sql.c' => {
         SQLEngine => 'log',
         SQLBackend => 'sqlite3',
-
-        # We provide dummy username/password values, in order to specify
-        # the policy.  In case of chroots, we need to explicitly use
-        # the PERCONNECTION policy.
-        SQLConnectInfo => "$db_file foo bar PERCONNECTION",
-
+        SQLConnectInfo => $db_file,
         SQLLogFile => $setup->{log_file},
         SQLNamedQuery => 'file_xfer FREEFORM "INSERT INTO proxy_transfers (user, protocol, frontend_ipaddr, local_ipaddr, backend_ipaddr, backend_port, file, timestamp) VALUES (\'%u\', \'%{protocol}\', \'%a\', \'%L\', \'%{note:mod_proxy.backend-ip}\', %{note:mod_proxy.backend-port}, \'%F\', \'%{iso8601}\')"',
         SQLLog => 'RETR,STOR file_xfer',
@@ -2148,7 +2113,7 @@ EOC
     my $expected = $setup->{user};
     $self->assert($expected eq $login, "Expected '$expected', got '$login'");
 
-    $expected = 'ftp';
+    my $expected = 'ftp';
     $self->assert($expected eq $protocol, "Expected '$expected', got '$protocol'");
 
     $expected = '127.0.0.1';
